#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/times.h>
#include <sys/wait.h>
#include <unistd.h>
#include <math.h>

#define EPSILON 0.001
long double current_n;

void error_fatal (const char *message){
  perror (message);
  exit (EXIT_FAILURE);
}

static void sig_alrm(int signo){
  fprintf (stderr, "Not enough time to calculate fibonaci number with index of %Lf\n", current_n);
  exit (EXIT_SUCCESS);
}

/* Funkcija racuna n-ti clan Fibonacijevog niza. Koristi se tip long
   double kako bi mogli da se racunaju veliki clanovi ovog niza. */
long double fib (long double n){
  /* Realni brojevi se ne mogu porediti pomocu jednakosti. */
  if (fabsl (n) < EPSILON || fabsl (n-1) < EPSILON)
    return 1.0;
  else {
    long double fib_1 = fib(n-1);
    long double fib_2 = fib(n-2);
    return fib_1 + fib_2;
  }
}

int main (int argc, char **argv){
  long double n, fib_n;
  FILE *in;
  
  /* Provera argumenata komandne linije. */
  if (argc != 2){
    fprintf (stderr, "Input file needed as only argument\n");
    exit (EXIT_FAILURE);
  }
  
  if (signal(SIGALRM, sig_alrm) == SIG_ERR)
    error_fatal ("signal error");

  if ( (in = fopen (argv[1], "r")) == NULL)
    error_fatal ("fopen error for reading");

  while (fscanf (in, "%Lf", &n) == 1){
    current_n = n;
    fib_n = fib (n);
    printf ("fib (%Lf) = %Lf\n", n, fib_n);
  }

  return 0;
}

